#include <windows.h>
#include <commctrl.h>
#include "main.h"

WNDPROC g_ListViewWndProc = NULL;
const char g_fileheader[] = "Pending File Move Operations:\r\n";
	
HWND InitListView(HWND hwnd, int idc)
{
	HWND hListView;
	LVCOLUMN lvc;
	RECT r;

	hListView = GetDlgItem(hwnd, idc);
	ListView_SetExtendedListViewStyle(hListView, 
			LVS_EX_FULLROWSELECT | 0x00004000 /* LVS_EX_LABELTIP */);

	GetClientRect(hListView, &r);

	lvc.mask = LVCF_SUBITEM | LVCF_WIDTH | LVCF_TEXT;
	lvc.cx = (r.right / 2);
	lvc.pszText = "Source";
	lvc.iSubItem = 0;
	ListView_InsertColumn(hListView, lvc.iSubItem, &lvc);

	lvc.cx = (r.right / 2) - GetSystemMetrics(SM_CXHSCROLL) - 2;
	lvc.pszText = "Target";
	lvc.iSubItem = 1;
	ListView_InsertColumn(hListView, lvc.iSubItem, &lvc);

	g_ListViewWndProc = (WNDPROC)SetWindowLongPtr(hListView, 
			GWL_WNDPROC, (LONG)EmptyTextListViewWndProc);

	SendMessage(hListView, LVUM_SETEMPTYTEXT, 0, 
			(LPARAM)"No pending file move operations found.");

	return hListView;
}

LRESULT SetEmptyText(HWND hListView, LPLONG lplvet, const char *emptytext)
{
	static LVEMPTYTEXT lvet;
	HFONT hFontOld;
	HDC hdc;

	if (emptytext == NULL)
	{
		*lplvet = 0;
		return TRUE;
	}

	hdc = GetDC(hListView);

	lvet.hFont = (HFONT)SendMessage(hListView, WM_GETFONT, 0, 0);
	hFontOld = (HFONT)SelectObject(hdc, lvet.hFont);

	DrawText(hdc, emptytext, -1, &lvet.rcMargins, DT_CALCRECT | DT_NOPREFIX);

	SelectObject(hdc, hFontOld);
	ReleaseDC(hListView, hdc);

	lvet.rcMargins.left = 0;
	lvet.rcMargins.top = 60;
	lvet.rcMargins.right = 0;
	lvet.rcMargins.bottom += 60;

	lstrcpy(lvet.szMarkup, emptytext);

	*lplvet = (LONG)&lvet;

	InvalidateEmptyText(hListView, &lvet, TRUE);

	return TRUE;
}

void InvalidateEmptyText(HWND hListView, LPLVEMPTYTEXT lplvet, BOOL setwidth)
{
	RECT rc;

	if (lplvet == NULL)
		return;

	if (ListView_GetItemCount(hListView) == 0)
	{
		if (setwidth == TRUE)
		{
			GetClientRect(hListView, &rc);
			if (lplvet->rcMargins.right == rc.right)
				return;

			lplvet->rcMargins.right = rc.right;
		}

		InvalidateRect(hListView, &lplvet->rcMargins, TRUE);
	}
}

LRESULT DrawEmptyText(HWND hListView, LPLVEMPTYTEXT lplvet, WPARAM wParam, LPARAM lParam)
{
	LRESULT ret = CallWindowProc(g_ListViewWndProc, hListView, WM_ERASEBKGND, wParam, lParam);

	if (lplvet == NULL)
		return ret;
	
	if (ListView_GetItemCount(hListView) == 0)
	{
		RECT rc;
		HFONT hFontOld;
		int iBkmOld;

		rc.left = lplvet->rcMargins.left + 10;
		rc.top = lplvet->rcMargins.top;
		rc.right = lplvet->rcMargins.right - 10;
		rc.bottom = lplvet->rcMargins.bottom;

		iBkmOld = SetBkMode((HDC)wParam, TRANSPARENT);
		hFontOld = (HFONT)SelectObject((HDC)wParam, lplvet->hFont);

		DrawText((HDC)wParam, lplvet->szMarkup, -1, &rc, DT_NOPREFIX | DT_CENTER);

		SelectObject((HDC)wParam, hFontOld);
		SetBkMode((HDC)wParam, iBkmOld);
	}

	return ret;
}

LRESULT CALLBACK EmptyTextListViewWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static LONG_PTR lplvet = 0;

  switch (uMsg)
  {
	case WM_ERASEBKGND:
		return DrawEmptyText(hwnd, (LPLVEMPTYTEXT)lplvet, wParam, lParam);

	case WM_NOTIFY:
	{
		if (wParam == 0)
		{
			switch (((NMHDR *)lParam)->code)
			{
				case HDN_ITEMCHANGINGA:
				case HDN_ITEMCHANGINGW:
//				case HDN_ITEMCHANGEDA:
//				case HDN_ITEMCHANGEDW:
				case HDN_ENDTRACKA:
				case HDN_ENDTRACKW:
					InvalidateEmptyText(hwnd, (LPLVEMPTYTEXT)lplvet, FALSE);
					break;
			}
		}
	}
	break;

	case WM_MOUSEWHEEL:
	case WM_HSCROLL:
	case WM_VSCROLL:
		InvalidateEmptyText(hwnd, (LPLVEMPTYTEXT)lplvet, FALSE);
		break;

	case WM_WINDOWPOSCHANGED:
		InvalidateEmptyText(hwnd, (LPLVEMPTYTEXT)lplvet, TRUE);
		break;

	case LVUM_SETEMPTYTEXT:
		return SetEmptyText(hwnd, &lplvet, (char *)lParam);
  }
  
  return CallWindowProc(g_ListViewWndProc, hwnd, uMsg, wParam, lParam);
}

void AddListViewItem(HWND hListView, const char *source, const char *target, BYTE valuesource)
{
	char delstr[] = "DELETE";
//	char buf[1024];
	unsigned int status = 0;
	DWORD error;
	LVITEM lvi;

	error = file_exists(source);

	if (error == ERROR_PATH_NOT_FOUND ||
			error == ERROR_FILE_NOT_FOUND)
		status = ST_NOTFOUND;

	if (error == ERROR_ACCESS_DENIED)
		status = ST_ACCDENIED;

	lvi.mask = LVIF_TEXT | LVIF_PARAM;
	lvi.iItem = 0x7FFFFFFF;
	lvi.iSubItem = 0;
	lvi.lParam = valuesource | (status << 8);
	lvi.pszText = (char *)source;
/*
	if (status == ST_NOTFOUND)
	{
		wsprintf(buf, "File not found: %s", source);
		lvi.pszText = buf;
	}
	else
		lvi.pszText = (char *)source;
*/
	lvi.iItem = ListView_InsertItem(hListView, &lvi);

	lvi.mask = LVIF_TEXT;
	lvi.iSubItem = 1;
	lvi.pszText = (target == NULL) ? delstr : (char *)target;
	ListView_SetItem(hListView, &lvi);
}

void SavePendingFileMoveItems(HWND hwnd, HWND hListView, const char *filename)
{
	HANDLE hFile;
	DWORD dwWritten;
	char Text[1024];
	LVITEM lvi;

	lvi.pszText = Text;
	lvi.cchTextMax = sizeof(Text);
	lvi.iItem = -1;

	hFile = CreateFile(filename, GENERIC_WRITE, 0, NULL, 
			CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

	if (hFile == INVALID_HANDLE_VALUE)
	{
		ErrorMessage(hwnd, "Could not open file for writing: %s\n\n%s", filename);
		return;
	}

	WriteFile(hFile, g_fileheader, sizeof(g_fileheader)-1, &dwWritten, NULL);

	while ((lvi.iItem = ListView_GetNextItem(hListView, lvi.iItem, LVNI_ALL)) != -1)
	{
		lvi.iSubItem = 0;
		WriteFile(hFile, "\r\nSource: ", 10, &dwWritten, NULL);

		if (SendMessage(hListView, LVM_GETITEMTEXT, lvi.iItem, (LPARAM)&lvi))
			WriteFile(hFile, Text, strlen(Text), &dwWritten, NULL);

		lvi.iSubItem = 1;
		WriteFile(hFile, "\r\nTarget: ", 10, &dwWritten, NULL);

		if (SendMessage(hListView, LVM_GETITEMTEXT, lvi.iItem, (LPARAM)&lvi))
			WriteFile(hFile, Text, strlen(Text), &dwWritten, NULL);

		WriteFile(hFile, "\r\n", 2, &dwWritten, NULL);
	}

	CloseHandle(hFile);
}

DWORD __countsize(HWND hListView)
{
	char Text[1024];
	DWORD size = 0;
	LVITEM lvi;

	lvi.pszText = Text;
	lvi.cchTextMax = sizeof(Text);
	lvi.iItem = -1;

	while ((lvi.iItem = ListView_GetNextItem(hListView, lvi.iItem, LVNI_ALL)) != -1)
	{
		lvi.iSubItem = 0;
		size += (10 + SendMessage(hListView, LVM_GETITEMTEXT, lvi.iItem, (LPARAM)&lvi));

		lvi.iSubItem = 1;
		size += (10 + SendMessage(hListView, LVM_GETITEMTEXT, lvi.iItem, (LPARAM)&lvi) + 2);
	}

	return size + 1;
}

void CopyPendingFileMoveItems(HWND hwnd, HWND hListView)
{
	LPTSTR lptstrCopy;
	HGLOBAL hglbCopy;
	LVITEM lvi;

	if (!OpenClipboard(hwnd))
		return;

	EmptyClipboard();

	hglbCopy = GlobalAlloc(GMEM_MOVEABLE, __countsize(hListView));
	if (hglbCopy == NULL)
	{
		CloseClipboard();
		return;
	}

	lptstrCopy = (LPTSTR)GlobalLock(hglbCopy);
	lptstrCopy[0] = '\0';

	lvi.pszText = lptstrCopy;
	lvi.cchTextMax = 1024;
	lvi.iItem = -1;

	while ((lvi.iItem = ListView_GetNextItem(hListView, lvi.iItem, LVNI_ALL)) != -1)
	{
		lvi.iSubItem = 0;

		lstrcpy(lvi.pszText, "\r\nSource: ");
		lvi.pszText += 10;

		lvi.pszText += SendMessage(hListView, LVM_GETITEMTEXT, lvi.iItem, (LPARAM)&lvi);

		lvi.iSubItem = 1;

		lstrcpy(lvi.pszText, "\r\nTarget: ");
		lvi.pszText += 10;

		lvi.pszText += SendMessage(hListView, LVM_GETITEMTEXT, lvi.iItem, (LPARAM)&lvi);

		*lvi.pszText = '\r';
		*(++lvi.pszText) = '\n';
		*(++lvi.pszText) = '\0';
	}

	GlobalUnlock(hglbCopy);

	SetClipboardData(CF_TEXT, hglbCopy);
	CloseClipboard();
}
