#include "resources.h"

extern HINSTANCE g_hInstance;

typedef struct MoveSizeStruct {
	int nIDItem;         // The identifier of the control in parent window to be moved / resized.
	LONG xanchor;        // The horizontal position or offset relative to the right side of the parent window.
	LONG yanchor;        // The vertical position or offset relative to the bottom of the parent window.
	LONG wanchor;        // The width of the control window or offset relative to the width of the parent window.
	LONG hanchor;        // The height of the control window or offset relative to the height of the parent window.
	BYTE anchorflags;    // The window size and position anchoring flags.
} MoveSize;

#define ANCHOR_LEFT      1        // Anchor the left side of the control to the left side of the parent window.
#define ANCHOR_RIGHT     2        // Anchor the right side of the control to the right side of the parent window.
#define ANCHOR_TOP       4        // Anchor the top of the control to the top of the parent window.
#define ANCHOR_BOTTOM    8        // Anchor the bottom of the control to the bottom of the parent window.
#define ANCHOR_TOPLEFT   (ANCHOR_LEFT|ANCHOR_TOP)
#define ANCHOR_PARENT    (ANCHOR_LEFT|ANCHOR_TOP|ANCHOR_RIGHT|ANCHOR_BOTTOM)

// http://blogs.msdn.com/b/oldnewthing/archive/2012/10/24/10362204.aspx
#define LVUM_GETEMPTYTEXT WM_APP+25
#define LVUM_SETEMPTYTEXT WM_APP+26

typedef struct LVEmptyTextStruct {
  char  szMarkup[1024];
  RECT  rcMargins;
  HFONT hFont;
} LVEMPTYTEXT, *LPLVEMPTYTEXT;

#define ST_NOTFOUND   1
#define ST_ACCDENIED  2

typedef enum {
  S_REGPFRO = 1,
  S_REGPFRO2,
  S_WININIT
} S_SOURCE;

INT_PTR CALLBACK MainDialogProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
INT_PTR CALLBACK PropertiesDialogProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK EmptyTextListViewWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

HWND InitSizeGrip(HWND hwnd);
void MoveSizeControls(HWND hwnd, int num, MoveSize *ControlMoveSizeStruct, LONG newsize);

HWND InitListView(HWND hwnd, int idc);
void AddListViewItem(HWND hListView, const char *source, const char *target, BYTE valuesource);
void SavePendingFileMoveItems(HWND hwnd, HWND hListView, const char *filename);
void CopyPendingFileMoveItems(HWND hwnd, HWND hListView);

LRESULT SetEmptyText(HWND hListView, LPLONG lplvet, const char *emptytext);
void InvalidateEmptyText(HWND hListView, LPLVEMPTYTEXT lplvet, BOOL setwidth);
LRESULT DrawEmptyText(HWND hListView, LPLVEMPTYTEXT lplvet, WPARAM wParam, LPARAM lParam);

void StartListingPendingFileMoveOperations(HWND hListView);
void PendingFileRenameOperations(HWND hListView);
LONG EnumeratePFROValue(HWND hListView, HKEY hKeySmrg, const char *value);
void WininitRenameOperations(HWND hListView);

void StartPFROChangeNotifyThread(HWND hListView);
DWORD WINAPI PFROChangeNotifyThread(LPVOID param);
void StopPFROChangeNotifyThread(void);

BOOL GetFileName(HWND hwnd, char *filename);
UINT_PTR CALLBACK OFNHookProc(HWND hdlg, UINT uiMsg, WPARAM wParam, LPARAM lParam);
BOOL myCenterWindow(HWND hwndChild, HWND hwndParent);
int file_exists(const char *path);
int mystrcmpn(const char *s1, const char *s2, int n);
void ErrorMessage(HWND hwnd, const char *f, const char *m);

DWORD pGetLongPathName(LPCTSTR lpszShortPath, LPTSTR lpszLongPath, DWORD cchBuffer);
BOOL pIsWow64Process(HANDLE hProcess, PBOOL Wow64Process);
BOOL pWow64DisableWow64FsRedirection(PVOID *OldValue);
BOOL pWow64RevertWow64FsRedirection(PVOID *OldValue);
void LoadFunctionEntryPoints();
