#include <windows.h>
#include <commctrl.h>
#include "main.h"

#define SESSIONMANAGERKEY "System\\CurrentControlSet\\Control\\Session Manager"
#define BUFSIZE 0x8000

HANDLE g_hEventChangeNotify = NULL;
HANDLE g_hThread = NULL;

void StartListingPendingFileMoveOperations(HWND hListView)
{
	OSVERSIONINFO osv;
	osv.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

	if (GetVersionEx(&osv) && (osv.dwPlatformId == VER_PLATFORM_WIN32_NT))
		StartPFROChangeNotifyThread(hListView);
	else
		WininitRenameOperations(hListView);
}

void PendingFileRenameOperations(HWND hListView)
{
	char value[] = "PendingFileRenameOperations\0";
	char error[1024];
	HKEY hKey;
	LONG ret;

	SendMessage(hListView, WM_SETREDRAW, FALSE, 0);
	ListView_DeleteAllItems(hListView);
	
	ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, SESSIONMANAGERKEY, 0, 
			KEY_QUERY_VALUE, &hKey);

	if (ret == ERROR_SUCCESS)
	{
		if ((ret = EnumeratePFROValue(hListView, hKey, value)) != ERROR_SUCCESS)
		{
			wsprintf(error, "Error reading value %s: %u", value, ret);
			MessageBox(GetParent(hListView), error, APP_NAME, MB_ICONERROR);
		}

		value[lstrlen(value)] = '2';

		if ((ret = EnumeratePFROValue(hListView, hKey, value)) != ERROR_SUCCESS)
		{
			wsprintf(error, "Error reading value %s: %u", value, ret);
			MessageBox(GetParent(hListView), error, APP_NAME, MB_ICONERROR);
		}

		value[lstrlen(value)-1] = '\0';
		RegCloseKey(hKey);
	}
	else
	{
		wsprintf(error, "Could not open HKLM\\%s key", SESSIONMANAGERKEY);
		MessageBox(GetParent(hListView), error, APP_NAME, MB_ICONERROR);
	}

	SendMessage(hListView, WM_SETREDRAW, TRUE, 0);
}

LONG EnumeratePFROValue(HWND hListView, HKEY hKeySmrg, const char *value)
{
	char *pSource, *pTarget;
	void *buffer = NULL;
	DWORD type, len = 0;
	BYTE source = 0;
	LONG ret;

	source = (value[lstrlen(value)-1] == '2') ? 
			S_REGPFRO2 : S_REGPFRO;

	ret = RegQueryValueEx(hKeySmrg, value, 
			NULL, NULL, NULL, &len);

	if (ret == ERROR_SUCCESS)
	{
		buffer = malloc(len);
		if (!buffer) return -1;

		ret = RegQueryValueEx(hKeySmrg, value, 
				NULL, &type, buffer, &len);

		if (ret == ERROR_SUCCESS && type == REG_MULTI_SZ)
		{
			char *szData = buffer;

			while (*szData)
			{
				if (!mystrcmpn(szData, "\\??\\", 4))
					szData = szData + 4;

				pSource = szData;

				szData = szData + strlen(szData) + 1;

				if (*szData == '\0')
					pTarget = NULL;
				else
				{
					if (!mystrcmpn(szData, "\\??\\", 4))
						szData = szData + 4;

					if (!mystrcmpn(szData, "!\\??\\", 5))
						szData = szData + 5;

					pTarget = szData;
				}

				AddListViewItem(hListView, pSource, pTarget, source);

				szData = szData + strlen(szData) + 1;
			}
		}
		free(buffer);
	}

	return (ret == ERROR_FILE_NOT_FOUND ? ERROR_SUCCESS : ret);
}

void WininitRenameOperations(HWND hListView)
{
	char wininit[1024];
	char *pSource, *pTarget;
	void *buffer = NULL;

	buffer = malloc(BUFSIZE);
	if (!buffer) return;

	SendMessage(hListView, WM_SETREDRAW, FALSE, 0);
	ListView_DeleteAllItems(hListView);

	GetWindowsDirectory(wininit, sizeof(wininit)-16);
	lstrcat(wininit, "\\wininit.ini");

	GetPrivateProfileString(NULL, NULL, NULL, buffer, 0, wininit);
	if (GetPrivateProfileSection("Rename", buffer, BUFSIZE-1, wininit))
	{
		char *szData = buffer;
		char *ptr;

		while (*szData)
		{
			if (*szData != ';')
			{
				ptr = szData;

				while ((*ptr != '\0') && (*ptr != '=')) ptr++;

				if (*ptr != '\0')
				{
					*ptr = '\0';

					pTarget = szData;

					szData = szData + strlen(szData) + 1;

					pSource = szData;

					if (!lstrcmpi(pTarget, "NUL"))
						pTarget = NULL;

					AddListViewItem(hListView, pSource, pTarget, S_WININIT);
				}
			}
			szData = szData + strlen(szData) + 1;
		}
	}

	SendMessage(hListView, WM_SETREDRAW, TRUE, 0);
	free(buffer);
}

void StartPFROChangeNotifyThread(HWND hListView)
{
	g_hThread = CreateThread(NULL, 0, PFROChangeNotifyThread, (LPVOID)hListView, 0, NULL);

	if (g_hThread == INVALID_HANDLE_VALUE)
		PendingFileRenameOperations(hListView);
}

DWORD WINAPI PFROChangeNotifyThread(LPVOID param)
{
	HKEY hKey;
	LONG ret;

	ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
			SESSIONMANAGERKEY, 0, KEY_NOTIFY, &hKey);

	g_hEventChangeNotify = CreateEvent(NULL, TRUE, FALSE, NULL);

	if (ret == ERROR_SUCCESS && g_hEventChangeNotify != NULL)
	{
		while (IsWindow((HWND)param))
		{
			PendingFileRenameOperations((HWND)param);
			Sleep(100);

			if (RegNotifyChangeKeyValue(hKey, FALSE, REG_NOTIFY_CHANGE_LAST_SET, 
					g_hEventChangeNotify, TRUE) != ERROR_SUCCESS) break;

			if (WaitForSingleObject(g_hEventChangeNotify, 
					INFINITE) == WAIT_FAILED) break;
		}

		CloseHandle(g_hEventChangeNotify);
		RegCloseKey(hKey);
	}
	else
	{
		PendingFileRenameOperations((HWND)param);
	}
	
	return 0;
}

void StopPFROChangeNotifyThread()
{
	if (g_hThread != INVALID_HANDLE_VALUE)
	{
		SetEvent(g_hEventChangeNotify);
		WaitForSingleObject(g_hThread, 1000);
		CloseHandle(g_hThread);
	}
}
