#include <windows.h>
#include "main.h"

static RECT ofp;

UINT_PTR CALLBACK OFNHookProc(HWND hdlg, UINT uiMsg, WPARAM wParam, LPARAM lParam);

// Save file dialog with LPOFNHOOKPROC hook procedure.
BOOL GetFileName(HWND hwnd, char *filename)
{
	OPENFILENAME ofn;
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize = sizeof(ofn);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFilter = "Text File (*.txt)\0*.txt\0\0";
	ofn.lpstrDefExt = "txt";
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrFile = filename;
	ofn.Flags = OFN_EXPLORER | OFN_ENABLESIZING | OFN_HIDEREADONLY | 
			OFN_ENABLEHOOK | OFN_OVERWRITEPROMPT;
	ofn.lpfnHook = (LPOFNHOOKPROC)OFNHookProc;

	if (!GetSaveFileName(&ofn))
		return FALSE;

	return TRUE;
}

// Hook procedure for open file dialog to center the dialog.
// In OFNHookProc the WM_INITDIALOG message, lParam is a pointer to an OPENFILENAME 
// structure containing the values specified when the dialog box was created.
UINT_PTR CALLBACK OFNHookProc(HWND hdlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
	if (uiMsg == (UINT)WM_NOTIFY &&
		 ((NMHDR *)lParam)->code == CDN_INITDONE)
	{
		// Check if this is the first time the open file dialog is opened.
		if (IsRectEmpty(&ofp))
		{
		OPENFILENAME *ofn = ((OFNOTIFY *)lParam)->lpOFN;
			myCenterWindow((HWND)GetParent(hdlg), (HWND)ofn->hwndOwner);
		}
		else
		{
			// Open file dialog has been open before => restore last dialog position.
			SetWindowPos((HWND)GetParent(hdlg), NULL, ofp.left, ofp.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
		}
		return 1;
	}

	if (uiMsg == (UINT)WM_DESTROY)
	{
		// Save the dialog position.
		GetWindowRect((HWND)GetParent(hdlg), &ofp);
		return 1;
	}
	return 0;
}

// My own version of the MS KB 140722 CenterWindow function
// http://support.microsoft.com/kb/140722
// Centers hwndChild window by the hwndParent window.
BOOL myCenterWindow(HWND hwndChild, HWND hwndParent)
{
	RECT rC, rP;
	int xNew, yNew;

	// Get child window height and width.
	GetWindowRect(hwndChild, &rC);

	// Get parent window height and width.
	GetWindowRect(hwndParent, &rP);

	// Calculate new X and Y position, and keep the window in the screen.
	xNew = rP.left + (((rP.right - rP.left) - (rC.right - rC.left)) / 2);
	if (xNew > GetSystemMetrics (SM_CXSCREEN))
		xNew = (GetSystemMetrics (SM_CXSCREEN)-(rC.right - rC.left));
	if (xNew < 0) xNew = 0;
	yNew = rP.top + (((rP.bottom - rP.top) - (rC.bottom - rC.top)) / 2);
	if (yNew > GetSystemMetrics (SM_CYSCREEN))
		yNew = (GetSystemMetrics (SM_CYSCREEN)-(rC.bottom - rC.top));
	if (yNew < 0) yNew = 0;

	// Set new position, and return.
	return SetWindowPos(hwndChild, NULL, xNew, yNew, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}

int file_exists(const char *path)
{
	DWORD error = ERROR_SUCCESS;
	BOOL isWow64 = FALSE;
	PVOID OldValue = NULL;
	
	// Disable fs redirection on 64-bit Windows
	pIsWow64Process(GetCurrentProcess(), &isWow64);

	if (isWow64)
		pWow64DisableWow64FsRedirection(OldValue);

	if (GetFileAttributes(path) == INVALID_FILE_ATTRIBUTES)
		error = GetLastError();

	if (isWow64)
		pWow64RevertWow64FsRedirection(OldValue);

	return error;
}

// Compares two strings up to specified number of characters
int mystrcmpn(const char *s1, const char *s2, int n)
{
    while (--n && (*s1 != '\0') && (*s2 != '\0') && (*s1 == *s2))
	{
		s1++;
		s2++;
	}
    return (*s1 - *s2);
}

void ErrorMessage(HWND hwnd, const char *f, const char *m)
{
	char buf[1024], error[512];
	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, 
			GetLastError(), 0, error, sizeof(error), NULL);

	wsprintf(buf, f, m, error);
	MessageBox(hwnd, buf, "Error", MB_ICONERROR);
}
