#include <windows.h>
#include <commctrl.h>
#include "main.h"

typedef DWORD (WINAPI *GLPN)(LPCTSTR, LPTSTR, DWORD);
typedef BOOL (WINAPI *IWOW64P)(HANDLE, PBOOL);
typedef BOOL (WINAPI *WOW64DWFR)(PVOID);
typedef BOOL (WINAPI *WOW64RWFR)(PVOID);

static GLPN g_pGetLongPathName = NULL;
static IWOW64P g_pIsWow64Process = NULL;
static WOW64DWFR g_pWow64DisableWow64FsRedirection = NULL;
static WOW64RWFR g_pWow64RevertWow64FsRedirection = NULL;

#define MOVESIZESNUM 3
MoveSize MainControlMoveSize[MOVESIZESNUM] = 
{
	{IDC_PENDINGLIST, 0, 0, 0, 0, ANCHOR_PARENT}, 
	{IDC_CLOSEBUTTON, 0, 0, 0, 0, ANCHOR_RIGHT | ANCHOR_BOTTOM}, 
//	{IDC_PENDINGNEW, 0, 0, 0, 0, ANCHOR_LEFT | ANCHOR_BOTTOM}, 
//	{IDC_PENDINGEDIT, 0, 0, 0, 0, ANCHOR_LEFT | ANCHOR_BOTTOM}, 
//	{IDC_PENDINGREMOVE, 0, 0, 0, 0, ANCHOR_LEFT | ANCHOR_BOTTOM}, 
	{IDC_SIZEGRIP, 0, 0, 0, 0, ANCHOR_RIGHT | ANCHOR_BOTTOM}, 
};

#define PROPMOVESIZESNUM 4
MoveSize PropControlMoveSize[PROPMOVESIZESNUM] = 
{
	{IDOK, 0, 0, 0, 0, ANCHOR_BOTTOM}, 
	{IDC_TARGETLABEL, 0, 0, 0, 0, ANCHOR_BOTTOM}, 
	{IDC_TARGETFILE, 0, 0, 0, 0, ANCHOR_BOTTOM}, 
	{IDC_TARGETSTATUS, 0, 0, 0, 0, ANCHOR_BOTTOM}, 
};

HINSTANCE g_hInstance;

void MoveSizeControls(HWND hwnd, int num, MoveSize *ControlMoveSizeStruct, LONG newsize)
{
	RECT rcMain, rc;
	HWND control = NULL;

	int i;
	for (i=0; i<num; i++)
	{
		MoveSize *movesize = &ControlMoveSizeStruct[i];
		control = GetDlgItem(hwnd, movesize->nIDItem);

		if (movesize->anchorflags == ANCHOR_TOPLEFT) continue;
		if (control == NULL) continue;

		if (newsize == 0xffffffff)
		{
			GetClientRect(hwnd, &rcMain);

			GetWindowRect(control, &rc);
			ScreenToClient(hwnd, (POINT *)&rc.left);
			ScreenToClient(hwnd, (POINT *)&rc.right);

			movesize->xanchor = rc.left;
			movesize->yanchor = rc.top;

			movesize->wanchor = (rc.right - rc.left);
			movesize->hanchor = (rc.bottom - rc.top);

			if (movesize->anchorflags & ANCHOR_RIGHT)
			{
				if (movesize->anchorflags & ANCHOR_LEFT)
					movesize->wanchor = rcMain.right - (rc.right - rc.left);
				else
					movesize->xanchor = rcMain.right - rc.left;
			}

			if (movesize->anchorflags & ANCHOR_BOTTOM)
			{
				if (movesize->anchorflags & ANCHOR_TOP)
					movesize->hanchor = rcMain.bottom - (rc.bottom - rc.top);
				else
					movesize->yanchor = rcMain.bottom - rc.top;
			}
		}
		else
		{
			rc.left = movesize->xanchor;
			rc.top = movesize->yanchor;
			rc.right = movesize->wanchor;
			rc.bottom = movesize->hanchor;

			if (movesize->anchorflags & ANCHOR_RIGHT)
			{
				if (movesize->anchorflags & ANCHOR_LEFT)
					rc.right = LOWORD(newsize)-movesize->wanchor;
				else
					rc.left = LOWORD(newsize)-movesize->xanchor;
			}

			if (movesize->anchorflags & ANCHOR_BOTTOM)
			{
				if (movesize->anchorflags & ANCHOR_TOP)
					rc.bottom = HIWORD(newsize)-movesize->hanchor;
				else
					rc.top = HIWORD(newsize)-movesize->yanchor;
			}

			MoveWindow(control, rc.left, rc.top, rc.right, rc.bottom, TRUE);
		}
	}
}

HWND InitSizeGrip(HWND hwnd)
{
	HWND hSizeGrip;
	int width, height;
	RECT rc;

	width  = GetSystemMetrics(SM_CXVSCROLL);
	height = GetSystemMetrics(SM_CYHSCROLL);

	GetClientRect(hwnd, &rc);

	hSizeGrip = CreateWindowEx(0, WC_SCROLLBAR, NULL, 
			WS_CHILD | WS_VISIBLE | SBS_SIZEGRIP | SBS_SIZEBOXBOTTOMRIGHTALIGN | WS_CLIPSIBLINGS, 
			rc.right-width, rc.bottom-height, width, height, hwnd, (HMENU)IDC_SIZEGRIP, g_hInstance, NULL);

	return hSizeGrip;
}

INT_PTR CALLBACK MainDialogProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  static HWND hListView, hSizeGrip;
  static HMENU hMenu, hPopup;
  static char filename[MAX_PATH];

  if (uMsg == WM_DESTROY)
  {
	  DestroyMenu(hMenu);
	  return TRUE;
  }

  switch (uMsg)
  {
	case WM_INITDIALOG:
	{
		filename[0] = '\0';

		hMenu = LoadMenu(g_hInstance, MAKEINTRESOURCE(IDM_POPUP));
		if (hMenu) hPopup = GetSubMenu(hMenu, 0);

		hSizeGrip = InitSizeGrip(hwnd);
		hListView = InitListView(hwnd, IDC_PENDINGLIST);
		MoveSizeControls(hwnd, MOVESIZESNUM, &*MainControlMoveSize, 0xffffffff);

		StartListingPendingFileMoveOperations(hListView);
		return FALSE;
	}
	
	case WM_SYSCOLORCHANGE:
	case 794: /* WM_THEMECHANGED */
	{
		MoveSizeControls(hwnd, MOVESIZESNUM, &*MainControlMoveSize, 0xffffffff);
		return TRUE;
	}

	case WM_GETMINMAXINFO:
	{
		((MINMAXINFO*)lParam)->ptMinTrackSize.x=300;
		((MINMAXINFO*)lParam)->ptMinTrackSize.y=150;
		return TRUE;
	}

	case WM_SIZE:
	{
		if ((wParam == SIZE_MAXHIDE) || (wParam == SIZE_MAXSHOW)) return TRUE;

		MoveSizeControls(hwnd, MOVESIZESNUM, &*MainControlMoveSize, lParam);
		ShowWindow(hSizeGrip, (wParam == SIZE_MAXIMIZED) ? SW_HIDE : SW_SHOW);

		return TRUE;
	}

	case WM_INITMENUPOPUP:
	{
		if ((HMENU)wParam == hPopup)
		{
			EnableMenuItem((HMENU)wParam, IDC_PENDINGSAVE, 
					ListView_GetItemCount(hListView) ? MF_ENABLED : MF_GRAYED);

			EnableMenuItem((HMENU)wParam, IDC_PENDINGCOPY, 
					ListView_GetItemCount(hListView) ? MF_ENABLED : MF_GRAYED);

			EnableMenuItem((HMENU)wParam, IDC_PROPERTIES, 
					ListView_GetSelectedCount(hListView) ? MF_ENABLED : MF_GRAYED);
		}
		return TRUE;
	}

	case WM_NOTIFY:
	{
		if (wParam == IDC_PENDINGLIST)
		{
			switch(((NMHDR *)lParam)->code)
			{
				case NM_CUSTOMDRAW:
				{
					switch (((NMCUSTOMDRAW *)lParam)->dwDrawStage)
					{
						case CDDS_PREPAINT:
							SetWindowLong(hwnd, DWL_MSGRESULT, CDRF_NOTIFYITEMDRAW);
							return TRUE;

						case CDDS_ITEMPREPAINT:
							if (((((NMCUSTOMDRAW *)lParam)->lItemlParam & 0x0000FF00) >> 8) == ST_NOTFOUND)
								((NMLVCUSTOMDRAW *)lParam)->clrTextBk = RGB(248,200,200);
							SetWindowLong(hwnd, DWL_MSGRESULT, CDRF_DODEFAULT);
							return TRUE;
					}
				}
				break;

				case NM_RETURN:
				case NM_DBLCLK:
					PostMessage(hwnd, WM_COMMAND, MAKELONG(IDC_PROPERTIES, 0), 0);
					return TRUE;
	
				case NM_RCLICK:
				{
					POINT pt;
					GetCursorPos(&pt);
					TrackPopupMenu(hPopup, 0, pt.x, pt.y, 0, hwnd, NULL);
				}
				return TRUE;
			}
		}
		return FALSE;
	}

	case WM_COMMAND:
	{
		switch(LOWORD(wParam))
		{
			case IDC_PENDINGSAVE:
			{
				if (ListView_GetItemCount(hListView) != 0)
				{
					if (GetFileName(hwnd, filename) == TRUE)
						SavePendingFileMoveItems(hwnd, hListView, filename);
				}
				return TRUE;
			}

			case IDC_PENDINGCOPY:
			{
				if (ListView_GetItemCount(hListView) != 0)
					CopyPendingFileMoveItems(hwnd, hListView);

				return TRUE;
			}

			case IDC_PROPERTIES:
			{
				if (ListView_GetSelectedCount(hListView) == 1)
				{
					DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_PROPERTIES), 
							hwnd, (DLGPROC)PropertiesDialogProc, (LPARAM)hListView);
				}
				return TRUE;
			}

			case IDC_CLOSEBUTTON:
				EndDialog(hwnd, 0);
				return TRUE;
		}
		return FALSE;
	}
	
	case WM_CLOSE:
	{
		EndDialog(hwnd, 0);
		return TRUE;
	}
  }
  return 0;
}

BOOL SetStatusTextControlText(HWND hwnd, int nDlgItem, const char *text)
{
	HWND control;
	int offset;
	RECT rc;

	control = GetDlgItem(hwnd, nDlgItem);
	GetWindowRect(control, &rc);
	offset = rc.bottom;

	control = GetDlgItem(hwnd, IDC_SOURCEFILE);
	GetWindowRect(control, &rc);
	offset = (offset - rc.bottom);

	GetWindowRect(hwnd, &rc);

	rc.right = (rc.right - rc.left);
	rc.bottom = (rc.bottom - rc.top);

	rc.top = (rc.top - (offset / 2));
	rc.bottom = (rc.bottom + offset);

	MoveWindow(hwnd, rc.left, rc.top, rc.right, rc.bottom, TRUE);

	return SetWindowText(GetDlgItem(hwnd, nDlgItem), text);
}

void DrawStatusTextControl(DRAWITEMSTRUCT *lpdis)
{
	char text[1024];

	if (lpdis->itemAction & ODA_DRAWENTIRE)
	{
		HPEN hPen;
		RECT rc;

		// Get control's text
		if (GetWindowText(lpdis->hwndItem, text, sizeof(text)) == 0)
			return;

		// Create a pen with the appropriate color
		hPen = CreatePen(PS_SOLID, 1, GetSysColor(COLOR_WINDOWTEXT));
		SelectObject(lpdis->hDC, (HGDIOBJ)hPen);

		// Set line starting position
		MoveToEx(lpdis->hDC, 5, 3, NULL);

		// Draw the lines
		LineTo(lpdis->hDC, 5, lpdis->rcItem.bottom / 2);
		LineTo(lpdis->hDC, 17, lpdis->rcItem.bottom / 2);

		rc.left = lpdis->rcItem.left + 20;
		rc.top = lpdis->rcItem.top;
		rc.right = lpdis->rcItem.right;
		rc.bottom = lpdis->rcItem.bottom;

		// Draw text
		DrawText(lpdis->hDC, text, -1, &rc, DT_NOPREFIX);
	}
}

INT_PTR CALLBACK PropertiesDialogProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch (uMsg)
  {
	case WM_INITDIALOG:
	{
		LVITEM lvi;
		char Text[1024];

		lvi.iItem = -1;
		lvi.pszText = Text;
		lvi.cchTextMax = sizeof(Text);

		MoveSizeControls(hwnd, PROPMOVESIZESNUM, &*PropControlMoveSize, 0xffffffff);

		if ((lvi.iItem = ListView_GetNextItem((HWND)lParam, lvi.iItem, LVNI_SELECTED)) != -1)
		{
			lvi.mask = LVIF_PARAM | LVIF_TEXT;
			lvi.iSubItem = 0;

			if (SendMessage((HWND)lParam, LVM_GETITEM, 0, (LPARAM)&lvi))
			{
				DWORD error;

				pGetLongPathName(Text, Text, sizeof(Text));
				SetWindowText(GetDlgItem(hwnd, IDC_SOURCEFILE), Text);

				if ((error = file_exists(Text)) != ERROR_SUCCESS)
				{
					FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 
							NULL, error, 0, Text, 128, NULL);

					SetStatusTextControlText(hwnd, IDC_SOURCESTATUS, Text);
				}
			}

			lvi.mask = LVIF_TEXT;
			lvi.iSubItem = 1;

			if (SendMessage((HWND)lParam, LVM_GETITEM, 0, (LPARAM)&lvi))
			{
				pGetLongPathName(Text, Text, sizeof(Text));
				SetWindowText(GetDlgItem(hwnd, IDC_TARGETFILE), Text);
			}
		}
		return TRUE;
	}

	case WM_SIZE:
	{
		MoveSizeControls(hwnd, PROPMOVESIZESNUM, &*PropControlMoveSize, lParam);
		return TRUE;
	}

	case WM_DRAWITEM:
	{
		switch (((DRAWITEMSTRUCT*)lParam)->CtlID)
		{
			case IDC_SOURCESTATUS:
			case IDC_TARGETSTATUS:
				DrawStatusTextControl((DRAWITEMSTRUCT *)lParam);
				break;
		}
		return TRUE;
	}

	case WM_COMMAND:
	{
		switch(LOWORD(wParam))
		{
			case IDCANCEL:
			case IDOK:
			{
				EndDialog(hwnd, TRUE);
				return TRUE;
			}
		}
	}
  }
  return 0;
}

DWORD pGetLongPathName(LPCTSTR lpszShortPath, LPTSTR lpszLongPath, DWORD cchBuffer)
{
	if (g_pGetLongPathName)
		return g_pGetLongPathName(lpszShortPath, lpszLongPath, cchBuffer);

	return 0;
}

BOOL pIsWow64Process(HANDLE hProcess, PBOOL Wow64Process)
{
	if (g_pIsWow64Process)
		return g_pIsWow64Process(hProcess, Wow64Process);

	return FALSE;
}

BOOL pWow64DisableWow64FsRedirection(PVOID *OldValue)
{
	if (g_pWow64DisableWow64FsRedirection)
		return g_pWow64DisableWow64FsRedirection(OldValue);

	return FALSE;
}

BOOL pWow64RevertWow64FsRedirection(PVOID *OldValue)
{
	if (g_pWow64RevertWow64FsRedirection)
		return g_pWow64RevertWow64FsRedirection(OldValue);

	return FALSE;
}

void LoadFunctionEntryPoints()
{
	HMODULE hModule;
	hModule = GetModuleHandle("kernel32.dll");
	
	if (hModule != NULL)
	{
		g_pGetLongPathName = (GLPN)
				GetProcAddress(hModule, "GetLongPathNameA");

		g_pIsWow64Process = (IWOW64P)
				GetProcAddress(hModule, "IsWow64Process");

		g_pWow64DisableWow64FsRedirection = (WOW64DWFR)
				GetProcAddress(hModule, "Wow64DisableWow64FsRedirection");

		g_pWow64RevertWow64FsRedirection = (WOW64RWFR)
				GetProcAddress(hModule, "Wow64RevertWow64FsRedirection");
	}
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR lpCmdLine, int nCmdShow)
{
	InitCommonControls();
	g_hInstance = hInstance;

	LoadFunctionEntryPoints();

	DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAIN), 0, MainDialogProc);
	StopPFROChangeNotifyThread();

	return 0;
}
